@@ -50,6 +50,17 @@ if sys.platform[:3].lower() == "win":
 def _n(path):
     return os.path.abspath(os.path.normpath(path)).lower()
 
+def quote_command(command):
+    print " ".join(command)
+    # Quote the program name, so it works even if it contains spaces
+    command = " ".join(['"%s"' % x for x in command])
+    if WIN32: 
+        # odd, but true: the windows cmd processor can't handle more than 
+        # one quoted item per string unless you add quotes around the 
+        # whole line. 
+        command = '"%s"' % command 
+    return command
+
 class AdjustedZopeCmd(zopectl.ZopeCmd):
 
     if WIN32:
@@ -60,39 +71,54 @@ class AdjustedZopeCmd(zopectl.ZopeCmd):
             return
 
         def do_install(self, arg):
-            program = "%s install" % self.options.servicescript
-            print program
+            program = quote_command(self.options.servicescript + ['install'])
             os.system(program)
 
         def do_remove(self, arg):
-            program = "%s remove" % self.options.servicescript
-            print program
+            program = quote_command(self.options.servicescript + ['remove'])
             os.system(program)
 
         def do_start(self, arg):
-            program = "%s start" % self.options.servicescript
-            print program
+            program = quote_command(self.options.servicescript + ['start'])
             os.system(program)
 
         def do_stop(self, arg):
-            program = "%s stop" % self.options.servicescript
-            print program
+            program = quote_command(self.options.servicescript + ['stop'])
             os.system(program)
 
         def do_restart(self, arg):
-            program = "%s restart" % self.options.servicescript
-            print program
+            program = quote_command(self.options.servicescript + ['restart'])
             os.system(program)
 
-        def get_startup_cmd(self, python, more):
-            # if we pass a win32 backslashed path as a ''-style
-            # string, the backslashes will act as escapes.
-            # Use r'' instead.
-            cmdline = ( '%s -c "from Zope2 import configure;'
-                        'configure(r\'%s\');' %
-                        (python, self.options.configfile)
-                        )
-            return cmdline + more + '\"'
+    def get_startup_cmd(self, python, more):
+        # If we pass the configuration filename as a win32
+        # backslashed path using a ''-style string, the backslashes
+        # will act as escapes.  Use r'' instead.
+        cmdline = ( '%s -c "from Zope2 import configure; '
+                    'configure(r\'%s\'); ' %
+                    (python, self.options.configfile)
+                    )
+        return cmdline + more + '\"'
+
+    def do_run(self, arg):
+        tup = arg.split(' ')
+        if not arg:
+            print "usage: run <script> [args]"
+            return
+
+        # If we pass the script filename as a win32 backslashed path
+        # using a ''-style string, the backslashes will act as
+        # escapes.  Use r'' instead.
+        #
+        # Remove -c and add script as sys.argv[0]
+        script = tup[0]
+        cmd = 'import sys; sys.argv.pop(); sys.argv.append(r\'%s\'); '  % script
+        if len(tup) > 1:
+            argv = tup[1:]
+            cmd += '[sys.argv.append(x) for x in %s]; ' % argv
+        cmd += 'import Zope2; app=Zope2.app(); execfile(r\'%s\')' % script
+        cmdline = self.get_startup_cmd(self.options.python, cmd)
+        self._exitstatus = os.system(cmdline)
 
     def do_console(self, arg):
         self.do_foreground(arg, debug=False)
@@ -110,18 +136,17 @@ class AdjustedZopeCmd(zopectl.ZopeCmd):
                 return
         program = self.options.program
         if debug:
-            program[1:1] = ["-X", 'debug-mode=on']
-            print " ".join(program)
-            # Quote the program name, so it works even if it contains spaces
-            program=['"%s"' % x for x in program]
-            command=" ".join(program)
-            if WIN32: 
-                # odd, but true: the windows cmd processor can't handle more than 
-                # one quoted item per string unless you add quotes around the 
-                # whole line. 
-                command = '"%s"' % command 
-
-            return os.system(command)
+            local_additions = []
+            if not program.count('-X'):
+                local_additions += ['-X']
+            if not program.count('debug-mode=on'):
+                local_additions += ['debug-mode=on']
+            program[1:1] = local_additions
+            command = quote_command(program)
+            try:
+                return os.system(command)
+            finally:
+                for addition in local_additions: program.remove(addition)
         else:
             os.execv(program[0], program)
 
@@ -229,9 +254,9 @@ def main(args=None):
 
     # Add the path to the zopeservice.py script, which is needed for some of the
     # Windows specific commands    
-    servicescript = os.path.join(options.configroot.instancehome, 'bin', 'zopeservice.py')
-    options.servicescript = '"%s" %s' % (options.python, servicescript)
-
+    servicescript = os.path.join(options.configroot.instancehome, 
+                                 'bin', 'zopeservice.py')
+    options.servicescript = [options.python, servicescript]
 
     # If no command was specified we go into interactive mode.
     if options.args:
